package com.backstopsolutions.activemqtest;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Enumeration;

import javax.annotation.Resource;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.EJBException;
import javax.ejb.MessageDriven;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hornetq.api.jms.HornetQJMSClient;
import org.jboss.ejb3.annotation.ResourceAdapter;

import com.backstopsolutions.activemqtest.JmsUtil;

@MessageDriven(name = "RepeaterBean",
        activationConfig =
              {
                 @ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue"),
                 @ActivationConfigProperty(propertyName = "destination", propertyValue = "queue/repeaterqueue"),
                 @ActivationConfigProperty(propertyName = "acknowledgeMode", propertyValue = "Auto-acknowledge")
              })
@TransactionManagement(value= TransactionManagementType.CONTAINER)
@TransactionAttribute(value= TransactionAttributeType.REQUIRED)
@ResourceAdapter("hornetq-ra.rar")
public class RepeaterBean implements MessageDrivenBean, MessageListener {
    Log logger = LogFactory.getLog(RepeaterBean.class);
    
    @Resource
    protected MessageDrivenContext ctx;

    @Resource(mappedName = "java:JmsXA")
    protected QueueConnectionFactory qcf;
    
//    @Resource(mappedName = "activemq/queue/logqueue")
//    protected Queue logQueue;

    public void setMessageDrivenContext(MessageDrivenContext messageDrivenContext) {
        logger.info("Setting context: " + messageDrivenContext);
        this.ctx = messageDrivenContext;
    }

    public void ejbRemove() throws EJBException {
    }
    
    public void onMessage(Message msg) {
        MapMessage mapMsg = (MapMessage) msg;
        try {
            repeatMessage(mapMsg);
        } catch (JMSException e) {
            logger.error("Failed to process message", e);
            sendLogMessage("Failed to process message: " + e.getMessage());
        }
    }
    
    public void repeatMessage(MapMessage msg) throws JMSException {
        QueueSender sender = null;
        QueueSession session = null;
        QueueConnection con = null;

        try {
            String targetQueue = msg.getStringProperty("targetQueue");
            Queue queue = HornetQJMSClient.createQueue(targetQueue);
            con = this.qcf.createQueueConnection();
            session = con.createQueueSession(true, Session.AUTO_ACKNOWLEDGE);
            sender = session.createSender(queue);
            MapMessage newMsg = session.createMapMessage();
            for(Enumeration e = msg.getMapNames(); e.hasMoreElements(); ) {
                String name = (String) e.nextElement();
                newMsg.setObject(name, msg.getObject(name));
            }
            sender.send(newMsg);
            sender.close();
            sender = null;
            sendLogMessage("Successfully processed map message");
        } finally {
            JmsUtil.closeAll(con, session, sender);
        }
    }

    public void sendLogMessage(String msg) {
        QueueSender sender = null;
        QueueSession session = null;
        QueueConnection con = null;
        
        logger.info("Publishing log message: " + msg);
        
        try {
            con = this.qcf.createQueueConnection();
            session = con.createQueueSession(true, Session.AUTO_ACKNOWLEDGE);
            Queue logQueue = HornetQJMSClient.createQueue("queue.logqueue");
            sender = session.createSender(logQueue);
            TextMessage textMsg = session.createTextMessage();
            textMsg.setText(msg);
            textMsg.setStringProperty("sourceServer", InetAddress.getLocalHost().getHostName());
            logger.debug("Sending message: " + textMsg + " of class: " + textMsg.getClass());
            sender.send(textMsg);
            logger.info("Log message \"" + msg + "\" sent!");
        } catch (JMSException e) {
            logger.error("Failed to send log message: " + msg, e);
        } catch (UnknownHostException uhe) {
            logger.error("Failed to get local host name", uhe);
        } finally {
            JmsUtil.closeAll(con, session, sender);
        }
    }
}
